function bytes = gewrite(IMG, FILENAME, H)
% bytes = gewrite(IMG, FILENAME, H)
%
% Write an image file in GE 5.x DPCM-compressed 16-bit image format.
%
% IMG is the image data, and FILENAME is the filename to write.
%
% H is a header structure, made up of a series of sub-header structures.
% writege currently only understands the following subheaders:
%
%  H.ih  - general image information
%  H.mri - information specific to the MRI image
%
% If H or any component of H is not specified, reasonable defaults will be used.
% Note that many values of H.ih (e.g. the height, width, and depth of the image)
% are ignored, and are set according to the actual IMG data on the command line.
%
% EXAMPLES
%
% If you want to save an image IMG in 'test.mr', and you want to specify the
% absolute 3D position of the image by setting the field H.mri.corners equal
% to an existing variable CORNERS, you can type
%
%  gewrite(IMG, 'test.mr', struct('mri', struct('corners', CORNERS)));
%
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare a file header which matches the desired file structure

% Create the header if not specified
if nargin<3,
  H = struct('ih', struct('magic', 'IMGF'));
elseif ~isfield(H, 'ih'),
  H.ih = struct('magic', 'IMGF');
elseif ~isfield(H.ih, 'magic'),
  H.ih.magic = 'IMGF';
end;
ih = H.ih;

% Here we specify the fields in ih which will not be the same as the input ih
ih.header_version = 3;
[ih.height, ih.width ] = size(IMG);
ih.depth = 16;
ih.compress = 3;

% Define the list of sub-headers
% We specify a non-zero length for a header only if we know how to write it
headers = { ...
'ih', 156, 0; ...
'id', 0, 0; ...
'unpack', 0, 0; ...
'compress', 0, 0; ...
'histo', 2068, 0; ...
'text', 0, 0; ...
'graphics', 0, 0; ...
'dbHdr', 0, 0; ...
'user', 0, 0; ...
'suite', 114, 0; ...
'exam', 1024, 0; ...
'series', 1020, 0; ...
'mri', 1022, 0; ... 
};
[ NHD, foo ] = size(headers);

% ih contains a pointer to each existing header except ih
POINTER = headers{1,2};
for i=2:NHD,
  % if the header is not specified in the input structure, don't write it
  if ~isfield(H, headers{i,1}), headers{i,2} = 0; end;

  % Store the pointer and header length
  headers{i,3} = POINTER;
  ih = setfield(ih, [ 'p_' headers{i,1} ], headers{i,3});
  ih = setfield(ih, [ 'l_', headers{i,1} ], headers{i,2});

  % Update the pointer
  POINTER = headers{i,2} + POINTER;
end;

% Save the final POINTER as the header length
ih.hdr_length = POINTER;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% Write out the file
% Open the specified file in write-only mode with big-endian data
fid=fopen(FILENAME,'w','b');
if fid<0,
  error(sprintf('Unable to open file %s for writing', FILENAME)),
end

% Write out zeros in the length of the header, so that we can then use fseeks to move about
fwrite(fid, zeros(1, ih.hdr_length), 'uint8');
fseek(fid, 0, 'bof');

% Write out header sections that we know about
for i=find( [headers{:,2}] > 0),
  switch headers{i,1},
    case 'ih', fields = fields_ih; subhd = ih;
    case 'histo', fields = fields_histo; subhd = H.histo;
    case 'suite', fields = fields_suite; subhd = H.suite;
    case 'exam', fields = fields_exam; subhd = H.exam;
    case 'series', fields = fields_series; subhd = H.series;
    case 'mri', fields = fields_mri(H.mri); subhd = H.mri;
    otherwise, disp(strcat('writege: You asked me to write a header I am unable to write: ',headers{i,1}));
  end;
  bytes = write_subheader(fid, subhd, fields, headers{i,3}, headers{i,2}, ih); 
end;

% 
% Write out the image data
%
% First, fseek to the desired file start
fseek(fid, ih.hdr_length, 'bof');
n = gewrite_image(fid,IMG,ih.width,ih.height,ih.compress);
bytes = bytes+n;

fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write out a subheader defined by the list of fields in FIELDS,
% using data in the header structure S.
%
function [ OFFSET ] = write_subheader(fid, s, fields, pointer, s_length, ih)

% Start with an offset of zero
OFFSET = 0;

% Find the number of fields
[ NUMFLDS, DIM ] = size(fields);

% Write out all of the header fields
for fi=1:NUMFLDS,
  % Make sure that the current offset is correct
  if OFFSET < fields{fi,2},
    OFFSET = OFFSET + fwrite(fid, zeros(1, fields{fi,2}-OFFSET), 'uint8');
  end
  if OFFSET > fields{fi,2},
    fseek(fid, fields{fi,2}-OFFSET, 'cof');
%    disp(sprintf('Adjusting offset from %d to %d for field %s',OFFSET, fields{fi,2}, fields{fi,1}));
    OFFSET = fields{fi,2};
  end
  
  % If field does not exist in s, set it, then write it out
  if ~isfield(s, fields{fi,1}), 
    if ischar(fields{fi,4}),
%      disp(sprintf('Using default value of %s for field %s', fields{fi,4}, fields{fi,1}));
    else,
%      disp([ 'Using default value of ', sprintf('%g ', fields{fi,4}), 'for ', fields{fi,1}]);
    end
    s = setfield(s, fields{fi,1}, fields{fi,4}); 
  end
  val = getfield(s, fields{fi,1});
  if ischar(val),
%    disp(sprintf('Writing value of %s for field %s', val, fields{fi,1}));
  else,
%    disp([ 'Writing value of ', sprintf('%g ', val), 'for ', fields{fi,1}]);
  end
  n = fwrite(fid, val, fields{fi,3});
  
  % Shift the offset appropriately, depending on the specified type
  switch fields{fi,3},
    case { 'double' },
      OFFSET = OFFSET + 8*n;
    case { 'int' 'uint' 'float' },
      OFFSET = OFFSET + 4*n;
    case { 'short', 'ushort' },
      OFFSET = OFFSET + 2*n;
    case { 'char', 'uchar' },
      OFFSET = OFFSET + n;
    otherwise,
      disp(sprintf('gewrite: Unknown value type %s used, file will be damaged', fields{fi,3}));
  end
end  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function bytes = gewrite_image(fid, img,width,height,compress)
% bytes = gewrite_image(fid, img,width,height,compress)
%
%   img - a copy of the image to write
%   height, width (scalars) - size of the image to be created
%   compress [1x1] 	- compression code:
%	1 - image is rectangular (not compressed or packed)
%	2 - image is packed (not implemented)
%	3 - image is compressed (DPCM)
%	4 - image is compacked (not implemented)
%


% Start gewrite_image with a large last_pixel,
% to force us to write the first pixel value using three bytes
last_pixel = 2^64;
bytes = 0;
img = max(0,min(2^16-1,round(img)));

if (compress==1 | compress==2),
  % If there is no compression, we just write out the image in row order
  fwrite(fid, img', 'uint16');

else,
  % Otherwise, we compress using DPCM
  for row=1:height,
    for column=1:width,
      delta = img(row,column) - last_pixel;
      
      if ((delta >= -2^6) & (delta < 2^6)),
	% Code a 7-bit delta, with a zero top bit
	if delta < 0, delta = delta + 2^7; end;
	fwrite(fid, delta, 'uint8');
	
      elseif ((delta >= -2^13) & (delta < 2^13)),
	% If more than 7 bits needed, code a 14-bit delta, with '10' top bits
	if delta < 0, delta = delta + 2^14; end
	delta = bitset(delta,16);
	fwrite(fid, delta, 'uint16');
	
      else,
	% If more than 14 bits needed, we code the whole word
	fwrite(fid, 128+64,'uint8');
	fwrite(fid, img(row,column), 'uint16');
      end;
      last_pixel = img(row,column);
    end;
  end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fields = fields_ih
% fields = fields_ih

% Define the header fields, their byte offsets, types, and default values
fields = { ...
    'magic', 0, 'char', 'IMGF'; ...
'hdr_length', 4, 'int', 156; ...
'width', 8, 'int', 3336; ...
'height', 12, 'int', 256; ...
'depth', 16, 'int', 256; ...
'compress', 20, 'int', 3; ...
'default_window', 24, 'int', 0; ...
'default_level', 28, 'int', 0; ...
'background', 32, 'int', 0; ...
'ovrflow', 36, 'int', 2^14; ...
'undflow', 40, 'int', 0; ...
'top_offset', 44, 'int', 0; ...
'bot_offset', 48, 'int', 0; ...
'header_version', 52, 'short', 3; ...
'checksum', 54, 'ushort', 0
'p_id', 56, 'int', 156; ...
'l_id', 60, 'int', 156; ...
'p_unpack', 64, 'int', 156; ...
'l_unpack', 68, 'int', 156; ...
'p_compress', 72, 'int', 156; ...
'l_compress', 76, 'int', 156; ...
'p_histo', 80, 'int', 156; ...
'l_histo', 84, 'int', 156; ...
'p_text', 88, 'int', 156; ...
'l_text', 92, 'int', 156; ...
'p_graphics', 96, 'int', 156; ...
'l_graphics', 100, 'int', 156; ...
'p_dbHdr', 104, 'int', 156; ...
'l_dbHdr', 108, 'int', 156; ...
'levelOffset', 112, 'int', 156; ...
'p_user', 116, 'int', 156; ...
'l_user', 120, 'int', 156; ...
'p_suite', 124, 'int', 156; ...
'l_suite', 128, 'int', 156; ...
'p_exam', 132, 'int', 156; ...
'l_exam', 136, 'int', 156; ...
'p_series', 140, 'int', 156; ...
'l_series', 144, 'int', 156; ...
'p_mri', 148, 'int', 156; ...
'l_mri', 152, 'int', 156; ...
};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Private function fields_histo
function fields = fields_histo

% Define the fields which we should try to write
fields = { ...
'version', 0, 'int', 0; ...
'sd', 4, 'float', 0; ...
'mean', 8, 'short', 0; ...
'min', 10, 'short', 0; ...
'max', 12, 'short', 0; ...
'first', 14, 'short', 0; ...
'region', 16, 'short', 0; ...
'length', 18, 'short', 1022; ...
'bins', 20, 'ushort', zeros(1,1022); ...
'hist_left', 2064, 'ushort', 0; ...
'hist_right', 2066, 'ushort', 0; ...
};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Private function fields_suite, visible only within this file
function fields = fields_suite

% Define the fields
fields = { ...
'id', 0, 'char', 0; ...
'uniq', 4, 'short', 0; ...
'diskid', 6, 'char', 0; ...
'prodid', 6, 'char', 0; ...
'verscre', 20, 'char', 0; ...
'verscur', 22, 'char', 0; ...
'checksum', 24, 'uint', 0; ...
};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Private function fields_exam, visible only within this file
function fields = fields_exam

% Define the fields
fields = { ...
'suid', 0, 'char', 0; ...
'uniq', 4, 'short', 0; ...
'diskid', 6, 'char', 0; ...
'no', 8, 'ushort', 0; ...
'hospname', 10, 'char', 0; ...
'detect', 44, 'short', 0; ...
'numcells', 46, 'int', 0; ...
'zerocell', 50, 'float', 0; ...
'cellspace', 54, 'float', 0; ...
'srctodet', 58, 'float', 0; ...
'srctoiso', 62, 'float', 0; ...
'tubetyp', 66, 'short', 0; ...
'dastyp', 68, 'int', 0; ...
'num_dcnk', 70, 'short', 0; ...
'dcn_len', 72, 'short', 0; ...
'dcn_density', 74, 'short', 0; ...
'dcn_stepsize', 76, 'short', 0; ...
'dcn_shiftcnt', 78, 'short', 0; ...
'magstrength', 80, 'int', 0; ...
'patid', 84, 'char', 0; ...
'patname', 97, 'char', 0; ...
'patage', 122, 'short', 0; ...
'patian', 124, 'short', 0; ...
'patsex', 126, 'short', 0; ...
'patweight', 128, 'int', 0; ...
'trauma', 132, 'short', 0; ...
'hist', 134, 'char', 0; ...
'reqnum', 195, 'char', 0; ...
'datetime', 208, 'int', 0; ...
'refphy', 212, 'char', 0; ...
'diagrad', 245, 'char', 0; ...
'op', 278, 'char', 0; ...
'desc', 282, 'char', 0; ...
'typ', 305, 'char', 0; ...
'format', 308, 'short', 0; ...
'firstaxtime', 310, 'double', 0; ...
'sysid', 318, 'char', 0; ...
'lastmod', 328, 'int', 0; ...
'protocolflag', 332, 'short', 0; ...
'alloc_key', 334, 'char', 0; ...
'delta_cnt', 348, 'int', 0; ...
'verscre', 352, 'char', 0; ...
'verscur', 354, 'char', 0; ...
'checksum', 356, 'uint', 0; ...
'complete', 360, 'int', 0; ...
'seriesct', 364, 'int', 0; ...
'numarch', 368, 'int', 0; ...
'numseries', 372, 'int', 0; ...
'series', 376, 'char', 0; ...
'numunser', 384, 'int', 0; ...
'unseries', 388, 'char', 0; ...
'toarchcnt', 396, 'int', 0; ...
'toarchive', 400, 'char', 0; ...
'prospcnt', 408, 'int', 0; ...
'prosp', 412, 'char', 0; ...
'modelnum', 420, 'int', 0; ...
'modelcnt', 424, 'int', 0; ...
'models', 428, 'char', 0; ...
'stat', 436, 'short', 0; ...
'uniq_sys_id', 438, 'char', 0; ...
'service_id', 454, 'char', 0; ...
'mobile_loc', 470, 'char', 0; ...
'study_uid', 474, 'char', 0; ...
'study_status', 506, 'short', 0; ...
};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Private function fields_series, visible only within this file
function fields = fields_series

% Define fields
fields = { ...
'suid', 0, 'char', 0; ...
'uniq', 4, 'short', 0; ...
'diskid', 6, 'char', 0; ...
'exno', 8, 'ushort', 0; ...
'no', 10, 'short', 0; ...
'datetime', 12, 'int', 0; ...
'actual_dt', 16, 'int', 0; ...
'desc', 20, 'char', 0; ...
'pr_sysid', 50, 'char', 0; ...
'pansysid', 59, 'char', 0; ...
'typ', 68, 'short', 0; ...
'source', 70, 'short', 0; ...
'plane', 72, 'short', 0; ...
'scan_type', 74, 'short', 0; ...
'position', 76, 'int', 0; ...
'entry', 80, 'int', 0; ...
'anref', 84, 'char', 0; ...
'lmhor', 88, 'float', 0; ...
'prtcl', 92, 'char', 0; ...
'contrast', 118, 'short', 0; ...
'start_ras', 120, 'char', 0; ...
'start_loc', 122, 'float', 0; ...
'end_ras', 126, 'char', 0; ...
'end_loc', 128, 'float', 0; ...
'pseq', 132, 'short', 0; ...
'sortorder', 134, 'short', 0; ...
'lndmrkcnt', 136, 'int', 0; ...
'nacq', 140, 'short', 0; ...
'xbasest', 142, 'short', 0; ...
'xbaseend', 144, 'short', 0; ...
'xenhst', 146, 'short', 0; ...
'xenhend', 148, 'short', 0; ...
'lastmod', 150, 'int', 0; ...
'alloc_key', 154, 'char', 0; ...
'delta_cnt', 168, 'int', 0; ...
'verscre', 172, 'char', 0; ...
'verscur', 174, 'char', 0; ...
'pds_a', 176, 'float', 0; ...
'pds_c', 180, 'float', 0; ...
'pds_u', 184, 'float', 0; ...
'checksum', 188, 'uint', 0; ...
'complete', 192, 'int', 0; ...
'numarch', 196, 'int', 0; ...
'imagect', 200, 'int', 0; ...
'numimages', 204, 'int', 0; ...
'images', 208, 'char', 0; ...
'numunimg', 216, 'int', 0; ...
'unimages', 220, 'char', 0; ...
'toarchcnt', 228, 'int', 0; ...
'toarchive', 232, 'char', 0; ...
'echo1_alpha', 240, 'float', 0; ...
'echo1_beta', 244, 'float', 0; ...
'echo1_window', 248, 'ushort', 0; ...
'echo1_level', 250, 'short', 0; ...
'echo2_alpha', 252, 'float', 0; ...
'echo2_beta', 256, 'float', 0; ...
'echo2_window', 260, 'ushort', 0; ...
'echo2_level', 262, 'short', 0; ...
'echo3_alpha', 264, 'float', 0; ...
'echo3_beta', 268, 'float', 0; ...
'echo3_window', 272, 'ushort', 0; ...
'echo3_level', 274, 'short', 0; ...
'echo4_alpha', 276, 'float', 0; ...
'echo4_beta', 280, 'float', 0; ...
'echo4_window', 284, 'ushort', 0; ...
'echo4_level', 286, 'short', 0; ...
'echo5_alpha', 288, 'float', 0; ...
'echo5_beta', 292, 'float', 0; ...
'echo5_window', 296, 'ushort', 0; ...
'echo5_level', 298, 'short', 0; ...
'echo6_alpha', 300, 'float', 0; ...
'echo6_beta', 304, 'float', 0; ...
'echo6_window', 308, 'ushort', 0; ...
'echo6_level', 310, 'short', 0; ...
'echo7_alpha', 312, 'float', 0; ...
'echo7_beta', 316, 'float', 0; ...
'echo7_window', 320, 'ushort', 0; ...
'echo7_level', 322, 'short', 0; ...
'echo8_alpha', 324, 'float', 0; ...
'echo8_beta', 328, 'float', 0; ...
'echo8_window', 332, 'ushort', 0; ...
'echo8_level', 334, 'short', 0; ...
'series_uid', 336, 'char', 0; ...
'landmark_uid', 368, 'char', 0; ...
'equipmnt_uid', 400, 'char', 0; ...
};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Private function fields_mri, visible only within this file
% Some of the 'default values' are set according to related fields given in the argument structure
function fields = fields_mri(s)

% Get out the 'corners' element, if it exists
if isfield(s, 'corners'),
  corners = s.corners;
else
  corners = zeros(4,3);
end

fields = { ...
'suid', 0, 'char', 0; ...
'uniq', 4, 'short', 0; ...
'diskid', 6, 'char', 0; ...
'exno', 8, 'ushort', 0; ...
'seno', 10, 'short', 0; ...
'no', 12, 'short', 0; ...
'datetime', 14, 'int', 0; ...
'actual_dt', 18, 'int', 0; ...
'sctime', 22, 'float', 0; ...
'slthick', 26, 'float', 0; ...
'imatrix', 30, 'short', [256 256]; ...
'dfov', 34, 'float', 0; ...
'dfov_rect', 38, 'float', 0; ...
'dim', 42, 'float', [240 240]; ...
'pixsize', 50, 'float', [0.9375 0.9375]; ...
'pdid', 58, 'char', 0; ...
'contrastIV', 72, 'char', 0; ...
'contrastOral', 89, 'char', 0; ...
'contmode', 106, 'short', 0; ...
'serrx', 108, 'short', 0; ...
'imgrx', 110, 'short', 0; ...
'screenformat', 112, 'short', 0; ...
'plane', 114, 'short', 0; ...
'scanspacing', 116, 'float', 0; ...
'compress', 120, 'short', 0; ...
'scouttype', 122, 'short', 0; ...
'loc_ras', 124, 'char', 'I'; ...
'loc', 126, 'float', 0; ...
'ctr', 130, 'float', [0 0 0]; ...
'norm', 142, 'float', [0 0 0]; ...
'tlhc', 154, 'float', corners(1,:); ...
'trhc', 166, 'float', corners(2,:); ...
'brhc', 178, 'float', corners(4,:); ...
'forimgrev', 190, 'char', 0; ...
'tr', 194, 'int', 0; ...
'ti', 198, 'int', 0; ...
'te', 202, 'int', 0; ...
'numecho', 210, 'short', 0; ...
'echonum', 212, 'short', 0; ...
'tbldlta', 214, 'float', 0; ...
'nex', 218, 'float', 0; ...
'contig', 222, 'short', 0; ...
'hrtrate', 224, 'short', 0; ...
'tdel', 226, 'int', 0; ...
'saravg', 230, 'float', 0; ...
'sarpeak', 234, 'float', 0; ...
'monsar', 238, 'short', 0; ...
'trgwindow', 240, 'short', 0; ...
'reptime', 242, 'float', 0; ...
'imgpcyc', 246, 'short', 0; ...
'xmtgain', 248, 'short', 0; ...
'rcvgains', 250, 'short', 0; ...
'mr_flip', 254, 'short', 0; ...
'mindat', 256, 'int', 0; ...
'cphase', 260, 'short', 0; ...
'swappf', 262, 'short', 0; ...
'pauseint', 264, 'short', 0; ...
'pausetime', 266, 'float', 0; ...
'obplane', 270, 'int', 0; ...
'slocfov', 274, 'int', 0; ...
'xmtfreq', 278, 'int', 0; ...
'autoxmtfreq', 282, 'int', 0; ...
'autoxmtgain', 286, 'short', 0; ...
'prescan', 288, 'short', 0; ...
'user_bitmap', 292, 'int', 0; ...
'cenfreq', 296, 'short', 0; ...
'imode', 298, 'short', 0; ...
'iopt', 300, 'int', 0; ...
'pseq', 304, 'short', 0; ...
'pseqmode', 306, 'short', 0; ...
'psdname', 308, 'char', 0; ...
'psd_datetime', 342, 'int', 0; ...
'psd_iname', 346, 'char', 0; ...
'ctyp', 360, 'short', 0; ...
'cname', 362, 'char', 0; ...
'surfctyp', 380, 'short', 0; ...
'surfcext', 382, 'short', 0; ...
'rawrunnum', 384, 'int', 0; ...
'cal_fldstr', 388, 'uint', 0; ...
'supp_tech', 392, 'short', 0; ...
'vbw', 394, 'float', 0; ...
'slquant', 398, 'short', 0; ...
'gpre', 400, 'short', 0; ...
'intr_del', 402, 'int', 0; ...
'user', 406, 'float', 0; ...
'alloc_key', 506, 'char', 0; ...
'lastmod', 520, 'int', 0; ...
'verscre', 524, 'char', 0; ...
'verscur', 526, 'char', 0; ...
'pds', 528, 'int', 0; ...
'checksum', 540, 'uint', 0; ...
'archived', 544, 'int', 0; ...
'complete', 548, 'int', 0; ...
'satbits', 552, 'short', 0; ...
'scic', 554, 'short', 0; ...
'satloc', 556, 'short', 0; ...
'satthick', 568, 'short', 0; ...
'flax', 574, 'short', 0; ...
'venc', 576, 'short', 0; ...
'thk_disclmr', 578, 'short', 0; ...
'ps_flag', 580, 'short', 0; ...
'ps_status', 582, 'short', 0; ...
'image_type', 584, 'short', 0; ...
'vas_collapse', 586, 'short', 0; ...
'usern', 588, 'float', 0; ...
'proj_alg', 596, 'short', 0; ...
'proj_name', 598, 'char', 0; ...
'rotation', 612, 'float', 0; ...
'thresh_min_max', 624, 'int', 0; ...
'echo_trn_len', 640, 'short', 0; ...
'frac_echo', 642, 'short', 0; ...
'prep_pulse', 644, 'short', 0; ...
'cphasenum', 646, 'short', 0; ...
'var_echo', 648, 'short', 0; ...
'ref_img', 650, 'char', 0; ...
'sum_img', 651, 'char', 0; ...
'img_window', 652, 'ushort', 0; ...
'img_level', 654, 'short', 0; ...
'slop_int', 656, 'int', 0; ...
'slop_float', 676, 'float', 0; ...
'slop_str', 696, 'char', 0; ...
'scanactno', 728, 'short', 0; ...
'vasflags', 730, 'short', 0; ...
'vencscale', 732, 'float', 0; ...
'integrity', 736, 'short', 0; ...
'fphase', 738, 'int', 0; ...
'freq_dir', 742, 'short', 0; ...
'vas_mode', 744, 'short', 0; ...
'image_uid', 746, 'char', 0; ...
'sop_uid', 778, 'char', 0; ...
'pscopts', 816, 'short', 0; ...
'asoffset', 818, 'short', 0; ...
'unoriginal', 824, 'short', 0; ...
'interleaves', 826, 'short', 0; ...
'effechospace', 828, 'short', 0; ...
'viewsperseg', 830, 'short', 0; ...
'rbpm', 832, 'short', 0; ...
'rtpoint', 834, 'short', 0; ...
'rcvrtype', 836, 'short', 0; ...
'dbdt', 838, 'float', 0; ...
'dbdtper', 842, 'float', 0; ...
'estdbdtper', 846, 'float', 0; ...
'estdbdtts', 850, 'float', 0; ...
'saravghead', 854, 'float', 0; ...
};
